@@ -15,8 +15,10 @@ from typing import (
     Set,
     Tuple,
     Type,
+    TypedDict,
     TypeVar,
     Union,
+    cast,
 )
 
 from pypika import Order, Query, Table
@@ -34,6 +36,7 @@ from tortoise.exceptions import (
     OperationalError,
     ParamsError,
 )
+from tortoise.expressions import Expression
 from tortoise.fields.base import Field
 from tortoise.fields.data import IntField
 from tortoise.fields.relational import (
@@ -47,7 +50,6 @@ from tortoise.fields.relational import (
     ReverseRelation,
 )
 from tortoise.filters import FilterInfoDict, get_filters_for_field
-from tortoise.functions import Function
 from tortoise.indexes import Index
 from tortoise.manager import Manager
 from tortoise.queryset import (
@@ -87,6 +89,12 @@ def prepare_default_ordering(meta: "Model.Meta") -> Tuple[Tuple[str, Order], ...
     return parsed_ordering
 
 
+class FkSetterKwargs(TypedDict):
+    _key: str
+    relation_field: str
+    to_field: str
+
+
 def _fk_setter(
     self: "Model",
     value: "Optional[Model]",
@@ -339,9 +347,9 @@ class MetaInfo:
         for key in self.fk_fields:
             _key = f"_{key}"
             fk_field_object: ForeignKeyFieldInstance = self.fields_map[key]  # type: ignore
-            relation_field = fk_field_object.source_field
+            relation_field = cast(str, fk_field_object.source_field)
             to_field = fk_field_object.to_field_instance.model_field_name
-            property_kwargs = dict(
+            property_kwargs: FkSetterKwargs = dict(
                 _key=_key,
                 relation_field=relation_field,
                 to_field=to_field,
@@ -388,8 +396,8 @@ class MetaInfo:
         # Create lazy one to one fields on model.
         for key in self.o2o_fields:
             _key = f"_{key}"
-            o2o_field_object: OneToOneFieldInstance = self.fields_map[key]  # type: ignore
-            relation_field = o2o_field_object.source_field
+            o2o_field_object = cast(OneToOneFieldInstance, self.fields_map[key])
+            relation_field = cast(str, o2o_field_object.source_field)
             to_field = o2o_field_object.to_field_instance.model_field_name
             property_kwargs = dict(
                 _key=_key,
@@ -440,10 +448,11 @@ class MetaInfo:
         # Create lazy M2M fields on model.
         for key in self.m2m_fields:
             _key = f"_{key}"
+            field_object = cast(ManyToManyFieldInstance, self.fields_map[key])
             setattr(
                 self._model,
                 key,
-                property(partial(_m2m_getter, _key=_key, field_object=self.fields_map[key])),
+                property(partial(_m2m_getter, _key=_key, field_object=field_object)),
             )
 
     def _generate_db_fields(self) -> None:
@@ -478,7 +487,7 @@ class MetaInfo:
 class ModelMeta(type):
     __slots__ = ()
 
-    def __new__(mcs, name: str, bases: Tuple[Type, ...], attrs: dict):
+    def __new__(mcs, name: str, bases: Tuple[Type, ...], attrs: dict) -> "ModelMeta":
         fields_db_projection: Dict[str, str] = {}
         fields_map: Dict[str, Field] = {}
         filters: Dict[str, FilterInfoDict] = {}
@@ -605,7 +614,12 @@ class ModelMeta(type):
         meta.pk_attr = pk_attr
         meta.pk = fields_map.get(pk_attr)  # type: ignore
         if meta.pk:
-            meta.db_pk_column = meta.pk.source_field or meta.pk_attr
+            if meta.pk.source_field:
+                meta.db_pk_column = meta.pk.source_field
+            elif isinstance(meta.pk, OneToOneFieldInstance):
+                meta.db_pk_column = f"{meta.pk_attr}_id"
+            else:
+                meta.db_pk_column = meta.pk_attr
         meta._inited = False
         if not fields_map:
             meta.abstract = True
@@ -667,7 +681,7 @@ class Model(metaclass=ModelMeta):
             else:
                 setattr(self, key, deepcopy(field_object.default))
 
-    def __setattr__(self, key, value):
+    def __setattr__(self, key, value) -> None:
         # set field value override async default function
         if hasattr(self, "_await_when_save"):
             self._await_when_save.pop(key, None)
@@ -773,7 +787,7 @@ class Model(metaclass=ModelMeta):
             raise TypeError("Model instances without id are unhashable")
         return hash(self.pk)
 
-    def __iter__(self):
+    def __iter__(self) -> Iterable[Tuple]:
         for field in self._meta.db_fields:
             yield field, getattr(self, field)
 
@@ -781,7 +795,7 @@ class Model(metaclass=ModelMeta):
         return type(other) is type(self) and self.pk == other.pk  # type: ignore
 
     def _get_pk_val(self) -> Any:
-        return getattr(self, self._meta.pk_attr)
+        return getattr(self, self._meta.pk_attr, None)
 
     def _set_pk_val(self, value: Any) -> None:
         setattr(self, self._meta.pk_attr, value)
@@ -841,7 +855,7 @@ class Model(metaclass=ModelMeta):
         return self
 
     @classmethod
-    def register_listener(cls, signal: Signals, listener: Callable):
+    def register_listener(cls, signal: Signals, listener: Callable) -> None:
         """
         Register listener to current model class for special Signal.
 
@@ -1011,7 +1025,7 @@ class Model(metaclass=ModelMeta):
             setattr(self, field, getattr(obj, field, None))
 
     @classmethod
-    def _choose_db(cls, for_write: bool = False):
+    def _choose_db(cls, for_write: bool = False) -> BaseDBAsyncClient:
         """
         Return the connection that will be used if this query is executed now.
 
@@ -1238,6 +1252,13 @@ class Model(metaclass=ModelMeta):
         """
         return cls._db_queryset(using_db).first()
 
+    @classmethod
+    def last(cls, using_db: Optional[BaseDBAsyncClient] = None) -> QuerySetSingle[Optional[Self]]:
+        """
+        Generates a QuerySet that returns the last record.
+        """
+        return cls._db_queryset(using_db).last()
+
     @classmethod
     def filter(cls, *args: Q, **kwargs: Any) -> QuerySet[Self]:
         """
@@ -1248,6 +1269,24 @@ class Model(metaclass=ModelMeta):
         """
         return cls._meta.manager.get_queryset().filter(*args, **kwargs)
 
+    @classmethod
+    def latest(cls, *orderings: str) -> QuerySetSingle[Optional[Self]]:
+        """
+        Generates a QuerySet with the filter applied that returns the last record.
+
+        :params orderings: Fields to order by.
+        """
+        return cls._meta.manager.get_queryset().latest(*orderings)
+
+    @classmethod
+    def earliest(cls, *orderings: str) -> QuerySetSingle[Optional[Self]]:
+        """
+        Generates a QuerySet with the filter applied that returns the first record.
+
+        :params orderings: Fields to order by.
+        """
+        return cls._meta.manager.get_queryset().earliest(*orderings)
+
     @classmethod
     def exclude(cls, *args: Q, **kwargs: Any) -> QuerySet[Self]:
         """
@@ -1259,7 +1298,7 @@ class Model(metaclass=ModelMeta):
         return cls._meta.manager.get_queryset().exclude(*args, **kwargs)
 
     @classmethod
-    def annotate(cls, **kwargs: Union[Function, Term]) -> QuerySet[Self]:
+    def annotate(cls, **kwargs: Union[Expression, Term]) -> QuerySet[Self]:
         """
         Annotates the result set with extra Functions/Aggregations/Expressions.
 
